@@ -14,7 +14,6 @@ import (
 	"golang.org/x/tools/internal/lsp/command"
 	"golang.org/x/tools/internal/lsp/protocol"
 	"golang.org/x/tools/internal/lsp/source"
-	"golang.org/x/tools/internal/span"
 )
 
 // LensFuncs returns the supported lensFuncs for go.mod files.
@@ -23,6 +22,7 @@ func LensFuncs() map[command.Command]source.LensFunc {
 		command.UpgradeDependency: upgradeLenses,
 		command.Tidy:              tidyLens,
 		command.Vendor:            vendorLens,
+		command.RunVulncheckExp:   vulncheckLenses,
 	}
 }
 
@@ -50,7 +50,7 @@ func upgradeLenses(ctx context.Context, snapshot source.Snapshot, fh source.File
 	upgradeTransitive, err := command.NewUpgradeDependencyCommand("Upgrade transitive dependencies", command.DependencyArgs{
 		URI:        uri,
 		AddRequire: false,
-		GoCmdArgs:  []string{"-u", "all"},
+		GoCmdArgs:  []string{"-d", "-u", "-t", "./..."},
 	})
 	if err != nil {
 		return nil, err
@@ -58,7 +58,7 @@ func upgradeLenses(ctx context.Context, snapshot source.Snapshot, fh source.File
 	upgradeDirect, err := command.NewUpgradeDependencyCommand("Upgrade direct dependencies", command.DependencyArgs{
 		URI:        uri,
 		AddRequire: false,
-		GoCmdArgs:  requires,
+		GoCmdArgs:  append([]string{"-d"}, requires...),
 	})
 	if err != nil {
 		return nil, err
@@ -81,10 +81,6 @@ func tidyLens(ctx context.Context, snapshot source.Snapshot, fh source.FileHandl
 	if err != nil || pm.File == nil {
 		return nil, err
 	}
-	if len(pm.File.Require) == 0 {
-		// Nothing to vendor.
-		return nil, nil
-	}
 	uri := protocol.URIFromSpanURI(fh.URI())
 	cmd, err := command.NewTidyCommand("Run go mod tidy", command.URIArgs{URIs: []protocol.DocumentURI{uri}})
 	if err != nil {
@@ -105,6 +101,10 @@ func vendorLens(ctx context.Context, snapshot source.Snapshot, fh source.FileHan
 	if err != nil || pm.File == nil {
 		return nil, err
 	}
+	if len(pm.File.Require) == 0 {
+		// Nothing to vendor.
+		return nil, nil
+	}
 	rng, err := moduleStmtRange(fh, pm)
 	if err != nil {
 		return nil, err
@@ -129,7 +129,7 @@ func moduleStmtRange(fh source.FileHandle, pm *source.ParsedModule) (protocol.Ra
 		return protocol.Range{}, fmt.Errorf("no module statement in %s", fh.URI())
 	}
 	syntax := pm.File.Module.Syntax
-	return lineToRange(pm.Mapper, fh.URI(), syntax.Start, syntax.End)
+	return source.LineToRange(pm.Mapper, fh.URI(), syntax.Start, syntax.End)
 }
 
 // firstRequireRange returns the range for the first "require" in the given
@@ -150,19 +150,31 @@ func firstRequireRange(fh source.FileHandle, pm *source.ParsedModule) (protocol.
 	if start.Byte == 0 || firstRequire.Start.Byte < start.Byte {
 		start, end = firstRequire.Start, firstRequire.End
 	}
-	return lineToRange(pm.Mapper, fh.URI(), start, end)
+	return source.LineToRange(pm.Mapper, fh.URI(), start, end)
 }
 
-func lineToRange(m *protocol.ColumnMapper, uri span.URI, start, end modfile.Position) (protocol.Range, error) {
-	line, col, err := m.Converter.ToPosition(start.Byte)
-	if err != nil {
-		return protocol.Range{}, err
+func vulncheckLenses(ctx context.Context, snapshot source.Snapshot, fh source.FileHandle) ([]protocol.CodeLens, error) {
+	pm, err := snapshot.ParseMod(ctx, fh)
+	if err != nil || pm.File == nil {
+		return nil, err
 	}
-	s := span.NewPoint(line, col, start.Byte)
-	line, col, err = m.Converter.ToPosition(end.Byte)
+	// Place the codelenses near the module statement.
+	// A module may not have the require block,
+	// but vulnerabilities can exist in standard libraries.
+	uri := protocol.URIFromSpanURI(fh.URI())
+	rng, err := moduleStmtRange(fh, pm)
 	if err != nil {
-		return protocol.Range{}, err
+		return nil, err
 	}
-	e := span.NewPoint(line, col, end.Byte)
-	return m.Range(span.New(uri, s, e))
+
+	vulncheck, err := command.NewRunVulncheckExpCommand("Run govulncheck", command.VulncheckArgs{
+		URI:     uri,
+		Pattern: "./...",
+	})
+	if err != nil {
+		return nil, err
+	}
+	return []protocol.CodeLens{
+		{Range: rng, Command: vulncheck},
+	}, nil
 }
